@@ -1,16 +1,16 @@
 # tangram a general purpose table toolkit for R
 # Copyright (C) 2017 Shawn Garbett
-# 
+#
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -37,11 +37,13 @@
 #' }
 ASTNode <- R6Class("ASTNode",
   public = list(
-    value  = "character",
+    value      = "character",
+    format     = "character",
     terms      = function()     { return(c(self))    },
     distribute = function()     { return(self)       },
     string     = function()     { return(self$value) },
-    reduce     = function(data) { return(self)       }
+    reduce     = function(data) { return(self)       },
+    set_format = function(x)    { self$format <- x   }
   )
 )
 
@@ -76,7 +78,6 @@ ASTNode <- R6Class("ASTNode",
 ASTVariable <- R6Class("ASTVariable",
   inherit = ASTNode,
   public  = list(
-    format = "character",
     type   = "character",
     data   = NULL,
     initialize = function(identifier, format=NA, type=NA)
