@@ -6,7 +6,6 @@
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
@@ -38,7 +37,7 @@ gsub_table <- list(
 # Defered till all escaped characters are handled
 #  c("\\\u005C", "\\\\textbackslash{}"),
   c("\\\\\\\u005E", "\\\\textasciicircum{}"),
-  c("\\\\\\\u005F", "\\\\_"),
+  c("\u005F", "\\_"),
   c("\\\\\\\u0060", "\\\\textasciigrave{}"),
   c("\\\u007C", "\\\\vert{}"),
   c("\\\\\\\u007E", "\\\\textasciitilde{}"),
@@ -60,11 +59,11 @@ gsub_table <- list(
   c("\u00AE", "\\\\textregistered{}"),
   c("\u00AF", "\\\\textasciimacron{}"),
   c("\u00B0", "\\\\textdegree{}"),
-  c("\u00B1", "\\\\pm{}"),
-  c("\u00B2", "{^2}"),
-  c("\u00B3", "{^3}"),
-  c("\u00B4", "\\\\textasciiacute{}"),
-  c("\u00B5", "\\\\mathrm{\\\\mu}"),
+  c("\u00B1", "$\\\\pm{}$"),
+  c("\u00B2", "${^2}$"),
+  c("\u00B3", "${^3}$"),
+  c("\u00B4", "$\\\\textasciiacute{}$"),
+  c("\u00B5", "$\\\\mathrm{\\\\mu}$"),
   c("\u00B6", "\\\\textparagraph{}"),
   c("\u00B7", "\\\\cdot{}"),
   c("\u00B8", "\\\\c{}"),
@@ -416,49 +415,49 @@ gsub_table <- list(
   c("\u03AE", "\\\\acute{\\\\eta}"),
   c("\u03AF", "\\\\acute{\\\\iota}"),
   c("\u03B0", "\\\\acute{\\\\ddot{\\\\upsilon}}"),
-  c("\u03B1", "\\\\alpha{}"),
-  c("\u03B2", "\\\\beta{}"),
-  c("\u03B3", "\\\\gamma{}"),
-  c("\u03B4", "\\\\delta{}"),
-  c("\u03B5", "\\\\epsilon{}"),
-  c("\u03B6", "\\\\zeta{}"),
-  c("\u03B7", "\\\\eta{}"),
-  c("\u03B8", "\\\\texttheta{}"),
-  c("\u03B9", "\\\\iota{}"),
-  c("\u03BA", "\\\\kappa{}"),
-  c("\u03BB", "\\\\lambda{}"),
-  c("\u03BC", "\\\\mu{}"),
-  c("\u03BD", "\\\\nu{}"),
-  c("\u03BE", "\\\\xi{}"),
-  c("\u03C0", "\\\\pi{}"),
-  c("\u03C1", "\\\\rho{}"),
-  c("\u03C2", "\\\\varsigma{}"),
-  c("\u03C3", "\\\\sigma{}"),
-  c("\u03C4", "\\\\tau{}"),
-  c("\u03C5", "\\\\upsilon{}"),
-  c("\u03C6", "\\\\varphi{}"),
-  c("\u03C7", "\\\\chi{}"),
-  c("\u03C8", "\\\\psi{}"),
-  c("\u03C9", "\\\\omega{}"),
-  c("\u03CA", "\\\\ddot{\\\\iota}"),
-  c("\u03CB", "\\\\ddot{\\\\upsilon}"),
-  c("\u03CC", "\\\\'{o}"),
-  c("\u03CD", "\\\\acute{\\\\upsilon}"),
-  c("\u03CE", "\\\\acute{\\\\omega}"),
-  c("\u03D0", "\\\\Pisymbol{ppi022}{87}"),
-  c("\u03D1", "\\\\textvartheta{}"),
-  c("\u03D2", "\\\\Upsilon{}"),
-  c("\u03D5", "\\\\phi{}"),
-  c("\u03D6", "\\\\varpi{}"),
-  c("\u03DA", "\\\\Stigma{}"),
-  c("\u03DC", "\\\\Digamma{}"),
-  c("\u03DD", "\\\\digamma{}"),
-  c("\u03DE", "\\\\Koppa{}"),
-  c("\u03E0", "\\\\Sampi{}"),
-  c("\u03F0", "\\\\varkappa{}"),
-  c("\u03F1", "\\\\varrho{}"),
-  c("\u03F4", "\\\\textTheta{}"),
-  c("\u03F6", "\\\\backepsilon{}"),
+  c("\u03B1", "$\\\\alpha{}$"),
+  c("\u03B2", "$\\\\beta{}$"),
+  c("\u03B3", "$\\\\gamma{}$"),
+  c("\u03B4", "$\\\\delta{}$"),
+  c("\u03B5", "$\\\\epsilon{}$"),
+  c("\u03B6", "$\\\\zeta{}$"),
+  c("\u03B7", "$\\\\eta{}$"),
+  c("\u03B8", "$\\\\texttheta{}$"),
+  c("\u03B9", "$\\\\iota{}$"),
+  c("\u03BA", "$\\\\kappa{}$"),
+  c("\u03BB", "$\\\\lambda{}$"),
+  c("\u03BC", "$\\\\mu{}$"),
+  c("\u03BD", "$\\\\nu{}$"),
+  c("\u03BE", "$\\\\xi{}$"),
+  c("\u03C0", "$\\\\pi{}$"),
+  c("\u03C1", "$\\\\rho{}$"),
+  c("\u03C2", "$\\\\varsigma{}$"),
+  c("\u03C3", "$\\\\sigma{}$"),
+  c("\u03C4", "$\\\\tau{}$"),
+  c("\u03C5", "$\\\\upsilon{}$"),
+  c("\u03C6", "$\\\\varphi{}$"),
+  c("\u03C7", "$\\\\chi{}$"),
+  c("\u03C8", "$\\\\psi{}$"),
+  c("\u03C9", "$\\\\omega{}$"),
+  c("\u03CA", "$\\\\ddot{\\\\iota}$"),
+  c("\u03CB", "$\\\\ddot{\\\\upsilon}$"),
+  c("\u03CC", "$\\\\'{o}$"),
+  c("\u03CD", "$\\\\acute{\\\\upsilon}$"),
+  c("\u03CE", "$\\\\acute{\\\\omega}$"),
+  c("\u03D0", "$\\\\Pisymbol{ppi022}{87}$"),
+  c("\u03D1", "$\\\\textvartheta{}$"),
+  c("\u03D2", "$\\\\Upsilon{}$"),
+  c("\u03D5", "$\\\\phi{}$"),
+  c("\u03D6", "$\\\\varpi{}$"),
+  c("\u03DA", "$\\\\Stigma{}$"),
+  c("\u03DC", "$\\\\Digamma{}$"),
+  c("\u03DD", "$\\\\digamma{}$"),
+  c("\u03DE", "$\\\\Koppa{}$"),
+  c("\u03E0", "$\\\\Sampi{}$"),
+  c("\u03F0", "$\\\\varkappa{}$"),
+  c("\u03F1", "$\\\\varrho{}$"),
+  c("\u03F4", "$\\\\textTheta{}$"),
+  c("\u03F6", "$\\\\backepsilon{}$"),
   c("\u0401", "\\\\cyrchar\\\\CYRYO{}"),
   c("\u0402", "\\\\cyrchar\\\\CYRDJE{}"),
   c("\u0403", "\\\\cyrchar{\\\\'\\\\CYRG}"),
@@ -840,7 +839,7 @@ gsub_table <- list(
   c("\u2214", "\\\\dotplus{}"),
   c("\u2216", "\\\\setminus{}"),
   c("\u2217", "\\\\textasteriskcentered{}"),
-  c("\u2218", "\\\\circ{}"),
+  c("\u2218", "$\\\\circ{}$"),
   c("\u2219", "\\\\bullet{}"),
   c("\u221A", "\\\\surd{}"),
   c("\u221D", "\\\\propto{}"),
@@ -883,7 +882,7 @@ gsub_table <- list(
   c("\u2245", "\\\\cong{}"),
   c("\u2246", "\\\\approxnotequal{}"),
   c("\u2247", "\\\\not\\\\cong{}"),
-  c("\u2248", "\\\\approx{}"),
+  c("\u2248", "$\\\\approx{}$"),
   c("\u2249", "\\\\not\\\\approx{}"),
   c("\u224A", "\\\\approxeq{}"),
   c("\u224B", "\\\\tildetrpl{}"),
@@ -898,70 +897,70 @@ gsub_table <- list(
   c("\u2251", "\\\\doteqdot{}"),
   c("\u2252", "\\\\fallingdotseq{}"),
   c("\u2253", "\\\\risingdotseq{}"),
-  c("\u2254", ":="),
-  c("\u2255", "=:"),
-  c("\u2256", "\\\\eqcirc{}"),
-  c("\u2257", "\\\\circeq{}"),
-  c("\u2259", "\\\\estimates{}"),
-  c("\u225A", "\\\\ElsevierGlyph{225A}"),
-  c("\u225B", "\\\\starequal{}"),
-  c("\u225C", "\\\\triangleq{}"),
-  c("\u225F", "\\\\ElsevierGlyph{225F}"),
-  c("\u2260", "\\\\not ="),
-  c("\u2261", "\\\\equiv{}"),
-  c("\u2262", "\\\\not\\\\equiv{}"),
-  c("\u2264", "\\\\leq{}"),
-  c("\u2265", "\\\\geq{}"),
-  c("\u2266", "\\\\leqq{}"),
-  c("\u2267", "\\\\geqq{}"),
-  c("\u2268", "\\\\lneqq{}"),
-  c("\u2269", "\\\\gneqq{}"),
-  c("\u226A", "\\\\ll{}"),
-  c("\u226B", "\\\\gg{}"),
-  c("\u226C", "\\\\between{}"),
-  c("\u226D", "\\\\not\\\\kern-0.3em\\\\times{}"),
-  c("\u226E", "\\\\not&lt;"),
-  c("\u226F", "\\\\not&gt;"),
-  c("\u2270", "\\\\not\\\\leq{}"),
-  c("\u2271", "\\\\not\\\\geq{}"),
-  c("\u2272", "\\\\lessequivlnt{}"),
-  c("\u2273", "\\\\greaterequivlnt{}"),
-  c("\u2274", "\\\\ElsevierGlyph{2274}"),
-  c("\u2275", "\\\\ElsevierGlyph{2275}"),
-  c("\u2276", "\\\\lessgtr{}"),
-  c("\u2277", "\\\\gtrless{}"),
-  c("\u2278", "\\\\notlessgreater{}"),
-  c("\u2279", "\\\\notgreaterless{}"),
-  c("\u227A", "\\\\prec{}"),
-  c("\u227B", "\\\\succ{}"),
-  c("\u227C", "\\\\preccurlyeq{}"),
-  c("\u227D", "\\\\succcurlyeq{}"),
-  c("\u227E", "\\\\precapprox{}"),
-  c("\u227F", "\\\\succapprox{}"),
-  c("\u2280", "\\\\not\\\\prec{}"),
-  c("\u2281", "\\\\not\\\\succ{}"),
-  c("\u2282", "\\\\subset{}"),
-  c("\u2283", "\\\\supset{}"),
-  c("\u2284", "\\\\not\\\\subset{}"),
-  c("\u2285", "\\\\not\\\\supset{}"),
-  c("\u2286", "\\\\subseteq{}"),
-  c("\u2287", "\\\\supseteq{}"),
-  c("\u2288", "\\\\not\\\\subseteq{}"),
-  c("\u2289", "\\\\not\\\\supseteq{}"),
-  c("\u228A", "\\\\subsetneq{}"),
-  c("\u228B", "\\\\supsetneq{}"),
-  c("\u228E", "\\\\uplus{}"),
-  c("\u228F", "\\\\sqsubset{}"),
-  c("\u2290", "\\\\sqsupset{}"),
-  c("\u2291", "\\\\sqsubseteq{}"),
-  c("\u2292", "\\\\sqsupseteq{}"),
-  c("\u2293", "\\\\sqcap{}"),
-  c("\u2294", "\\\\sqcup{}"),
-  c("\u2295", "\\\\oplus{}"),
-  c("\u2296", "\\\\ominus{}"),
-  c("\u2297", "\\\\otimes{}"),
-  c("\u2298", "\\\\oslash{}"),
-  c("\u2299", "\\\\odot{}"),
+  c("\u2254", "$:=$"),
+  c("\u2255", "$=:$"),
+  c("\u2256", "$\\\\eqcirc{}$"),
+  c("\u2257", "$\\\\circeq{}$"),
+  c("\u2259", "$\\\\estimates{}$"),
+  c("\u225A", "$\\\\ElsevierGlyph{225A}$"),
+  c("\u225B", "$\\\\starequal{}$"),
+  c("\u225C", "$\\\\triangleq{}$"),
+  c("\u225F", "$\\\\ElsevierGlyph{225F}$"),
+  c("\u2260", "$\\\\not =$"),
+  c("\u2261", "$\\\\equiv{}$"),
+  c("\u2262", "$\\\\not\\\\equiv{}$"),
+  c("\u2264", "$\\\\leq{}$"),
+  c("\u2265", "$\\\\geq{}$"),
+  c("\u2266", "$\\\\leqq{}$"),
+  c("\u2267", "$\\\\geqq{}$"),
+  c("\u2268", "$\\\\lneqq{}$"),
+  c("\u2269", "$\\\\gneqq{}$"),
+  c("\u226A", "$\\\\ll{}$"),
+  c("\u226B", "$\\\\gg{}$"),
+  c("\u226C", "$\\\\between{}$"),
+  c("\u226D", "$\\\\not\\\\kern-0.3em\\\\times{}$"),
+  c("\u226E", "$\\\\not&lt;$"),
+  c("\u226F", "$\\\\not&gt;$"),
+  c("\u2270", "$\\\\not\\\\leq{}$"),
+  c("\u2271", "$\\\\not\\\\geq{}$"),
+  c("\u2272", "$\\\\lessequivlnt{}$"),
+  c("\u2273", "$\\\\greaterequivlnt{}$"),
+  c("\u2274", "$\\\\ElsevierGlyph{2274}$"),
+  c("\u2275", "$\\\\ElsevierGlyph{2275}$"),
+  c("\u2276", "$\\\\lessgtr{}$"),
+  c("\u2277", "$\\\\gtrless{}$"),
+  c("\u2278", "$\\\\notlessgreater{}$"),
+  c("\u2279", "$\\\\notgreaterless{}$"),
+  c("\u227A", "$\\\\prec{}$"),
+  c("\u227B", "$\\\\succ{}$"),
+  c("\u227C", "$\\\\preccurlyeq{}$"),
+  c("\u227D", "$\\\\succcurlyeq{}$"),
+  c("\u227E", "$\\\\precapprox{}$"),
+  c("\u227F", "$\\\\succapprox{}$"),
+  c("\u2280", "$\\\\not\\\\prec{}$"),
+  c("\u2281", "$\\\\not\\\\succ{}$"),
+  c("\u2282", "$\\\\subset{}$"),
+  c("\u2283", "$\\\\supset{}$"),
+  c("\u2284", "$\\\\not\\\\subset{}$"),
+  c("\u2285", "$\\\\not\\\\supset{}$"),
+  c("\u2286", "$\\\\subseteq{}$"),
+  c("\u2287", "$\\\\supseteq{}$"),
+  c("\u2288", "$\\\\not\\\\subseteq{}$"),
+  c("\u2289", "$\\\\not\\\\supseteq{}$"),
+  c("\u228A", "$\\\\subsetneq{}$"),
+  c("\u228B", "$\\\\supsetneq{}$"),
+  c("\u228E", "$\\\\uplus{}$"),
+  c("\u228F", "$\\\\sqsubset{}$"),
+  c("\u2290", "$\\\\sqsupset{}$"),
+  c("\u2291", "$\\\\sqsubseteq{}$"),
+  c("\u2292", "$\\\\sqsupseteq{}$"),
+  c("\u2293", "$\\\\sqcap{}$"),
+  c("\u2294", "$\\\\sqcup{}$"),
+  c("\u2295", "$\\\\oplus{}$"),
+  c("\u2296", "$\\\\ominus{}$"),
+  c("\u2297", "$\\\\otimes{}$"),
+  c("\u2298", "$\\\\oslash{}$"),
+  c("\u2299", "$\\\\odot{}$"),
   c("\u229A", "\\\\circledcirc{}"),
   c("\u229B", "\\\\circledast{}"),
   c("\u229D", "\\\\circleddash{}"),
@@ -2486,10 +2485,17 @@ latexify <- function(x)
   {
     if( (i %% 2) == 0)
     {
-      pieces[i] <- paste0("\\[", pieces[i], "\\]")
+      pieces[i] <- paste0("$", pieces[i], "$")
     } else # Odd, run gsub conversions
     {
-      for (subst in gsub_table) pieces[i] <- gsub(subst[1], subst[2], pieces[i], perl = TRUE)
+      #for (subst in gsub_table) pieces[i] <- gsub(subst[1], subst[2], pieces[i], perl = TRUE)
+      for (subst in gsub_table)
+      {
+        # Helpful debug code for latexify
+        #before <- pieces[i]
+        pieces[i] <- gsub(subst[1], subst[2], pieces[i], perl = TRUE)
+        #if(before != pieces[i]) cat("before ", before, " after ", pieces[i], " ", subst[1], "\n")
+      }
     }
   }
   y <- paste0(pieces, collapse="")
