@@ -15,12 +15,32 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #######
-# Given the compiled tree of data, render as a text rmd
+#' Generate an Rmd table entry from a cell object
+#'
+#' Given a cell object generate the corresponding piece of an Rmd table
+#'
 #' @include compile-cell.R
+#' @param object The cell_fstat for indexing
+#' @param key A filename to write key values into. Can be false if no key file is desired.
+#' @param append logical; Should the key file be appended too, or overwritten
+#' @param ... additional arguments to renderer. Unused
+#' @return A string representation of the table
+#' @rdname rmd
+#' @export
+#' @examples
+#'  rmd(tangram(drug ~ bili, pbc))
+rmd <- function(object, key=FALSE, ...)
+{
+  UseMethod("rmd", object)
+}
 
-rmd.default <- function(object,...) paste0(as.character(object), collapse=", ")
+#' @rdname rmd
+#' @export
+rmd.default <- function(object, key=FALSE, ...) paste0(as.character(object), collapse=", ")
 
-rmd.cell <- function(object, ...)
+#' @export
+#' @rdname rmd
+rmd.cell <- function(object, key=FALSE, ...)
 {
   sep  <- if(is.null(attr(object, "sep"))) ", " else attr(object, "sep")
 
@@ -33,65 +53,104 @@ rmd.cell <- function(object, ...)
   }
 }
 
-rmd.cell_iqr <- function(object,...)
+#' @export
+#' @rdname rmd
+rmd.cell_iqr <- function(object, key=FALSE, ...)
 {
-  paste0(object[1],
-         " **", object[2], "** ",
-         object[3])
+  if(key)
+  {
+    idx <- index(object, ...)
+    paste0("((",    object[1], "))%", word_ref(idx[[1]]), "%",
+           " **((", object[2], "))%", word_ref(idx[[2]]), "%** ",
+           "((",    object[3], "))%", word_ref(idx[[3]]), "%"
+           )
+  } else
+  {
+    paste0(object[1],
+           " **", object[2], "** ",
+           object[3])
+  }
 }
 
-
-rmd.cell_estimate <- function(object,...)
+#' @export
+#' @rdname rmd
+rmd.cell_estimate <- function(object, key=FALSE, ...)
 {
   paste0("(", rmd(object[1]), ", ", rmd(object[2]), ")")
 }
 
-rmd.cell_fstat <- function(object,...)
+#' @export
+#' @rdname rmd
+rmd.cell_fstat <- function(object, key=FALSE, ...)
 {
-  paste0("F<sub>",object[2],",",object[3],"</sub>=",object[1],", P=",object[4])
+  paste0("F~",object[2],",",object[3],"~=",object[1],", P=",object[4])
 }
 
-rmd.cell_fraction <- function(object,...)
+#' @export
+#' @rdname rmd
+rmd.cell_fraction <- function(object, key=FALSE, ...)
 {
   den <- object["denominator"]
-  num <- paste0(rep("", nchar(den) - nchar(object["numerator"])), object["numerator"])
+  num <- object["numerator"]
   paste0(object["ratio"], "  ", num,"/",den)
 }
 
-rmd.cell_chi2 <- function(object,...)
+#' @export
+#' @rdname rmd
+rmd.cell_chi2 <- function(object, key=FALSE, ...)
 {
-  paste0("\u03a7^2^~", object[2], "~=", object[1], ", P=", object[3])
+  if(key)
+  {
+    idx <- index(object, ...)
+    paste0("X^2^((~", object[2], "~))%", word_ref(idx[[2]]), "%",
+           "=((", object[1], "))%",    word_ref(idx[[1]]), "%",
+           ", P=((", object[3], "))%",  word_ref(idx[[3]]), "%")
+  } else
+  {
+    paste0("X^2^~", object[2],
+           "~=", object[1],
+           ", P=", object[3])
+  }
 }
 
-rmd.cell_studentt <- function(object,...)
+#' @export
+#' @rdname rmd
+rmd.cell_studentt <- function(object, key=FALSE, ...)
 {
+  idx <- index(object, key=FALSE, ...)
+
   paste0("T~",object[2],"~=",object[1], ", P=",object[3])
 }
 
-rmd.cell_spearman <- function(object,...)
+#' @export
+#' @rdname rmd
+rmd.cell_spearman <- function(object, key=FALSE, ...)
 {
   paste0("S=",object[1],", P=",object[1])
 }
 
-rmd.cell_n <- function(object,...)
+#' @export
+#' @rdname rmd
+rmd.cell_n <- function(object, key=FALSE, ...)
 {
-  if (inherits(object, "cell_header"))
-    paste0("(N=",as.character(object),")")
-  else
+  rep <- if(key)
+  {
+    idx <- index(object, ...)
+    paste0("((", as.character(object), "))%", word_ref(idx[[1]]), "%")
+  } else
+  {
     as.character(object)
+  }
+  if (inherits(object, "cell_header")) paste0("(N=",rep,")") else rep
 }
 
-#' Generate an Rmd table entry from a tangram object
-#'
-#' Given a tangram object generate the corresponding piece of an Rmd table
-#'
-#' @param object The cell_fstat for indexing
-#' @param ... additional arguments to renderer. Unused
-#' @return A string representation of the table
+
+#' @rdname rmd
 #' @export
 #'
 #' @importFrom stringr str_pad
-rmd.tangram <- function(object,...)
+#' @importFrom utils write.table
+rmd.tangram <- function(object, key=NULL, append=FALSE, ...)
 {
   nrows <- rows(object)
   ncols <- cols(object)
@@ -102,7 +161,7 @@ rmd.tangram <- function(object,...)
   sapply(1:nrows, FUN=function(row) {
     sapply(1:ncols, FUN=function(col) {
       if(last_header_row == 0 && !inherits(object[[row]][[col]], "cell_header")) last_header_row <<- row - 1
-      text[row,col] <<- rmd(object[[row]][[col]])
+      text[row,col] <<- rmd(object[[row]][[col]], key=!is.null(key), ...)
     })
   })
 
@@ -117,6 +176,8 @@ rmd.tangram <- function(object,...)
     }
   })
 
+  cat('\n') # An RMarkdown table must be preceeded by a newline or bad things happen
+
   pasty <- apply(text, 1, function(x) paste(c("|", paste(x, collapse="|"), "|"), collapse=""))
 
   cat(pasty[1], '\n')
@@ -124,27 +185,14 @@ rmd.tangram <- function(object,...)
 
   for(row in pasty[2:nrows]) cat(row, '\n')
 
+  if(!is.null(key)) write.table(index(object, ...), key, col.names=FALSE, row.names=FALSE, append=append, sep=",", quote=FALSE)
 }
 
-#' Generate an Rmd table entry from a cell object
-#'
-#' Given a cell object generate the corresponding piece of an Rmd table
-#'
-#' @param object The cell_fstat for indexing
-#' @param ... additional arguments to renderer. Unused
-#' @return A string representation of the table
+#' @rdname rmd
 #' @export
-#'
-rmd <- function(object, ...)
+rmd.table_builder <- function(object, key=FALSE, ...)
 {
-  UseMethod("rmd", object)
-}
-
-#' @rdname summary
-#' @export
-rmd.table_builder <- function(object,...)
-{
-  rmd(table_flatten(object$table))
+  rmd(table_flatten(object$table), key=key, ...)
 }
 
 
